#include "scanline.h"
#define DEBUGP	0

typedef struct
{
	int *scanline1,*scanline2;
	int *scanlinev;
	int *lens1,*lens2,*runs1,*runs2,*num_runs1,*num_runs2,*corr1,*corr2,*morph_lens;
	unsigned char *buffer1,*buffer2,*morph_buffer;
}Scanline;

Scanline scanline;

void print(char *buffer)
{
	if(DEBUGP)
		printf("%s\n",buffer);
}

void printMatrix(CvMatrix3 *m)
{
	int i,j;
	for(i=0;i<3;i++)
	{
		printf("[");
		for(j=0;j<3;j++)
			printf("%g ",m->m[i][j]);
		printf("]\n");
	}
}

int Scanlines(CvPoint2D64d *Right,CvPoint2D64d *Left,int *size,int w,int h)
{
	int i;
	CvMatrix3 *matrix;
	CvMat *cPoint1,*cPoint2,*mat,*status;
	cPoint1 = cvCreateMat(2,(*size)*2,CV_32F);
	cPoint2 = cvCreateMat(2,(*size)*2,CV_32F);
	status = cvCreateMat(1,*(size)*2,CV_32F);
	mat = cvCreateMat(3,3,CV_32F);
	for(i=0;i<(*size);i++)
	{
		cvmSet(cPoint1,0,i,(double)Right[i].x);
		cvmSet(cPoint1,1,i,(double)Right[i].y);
		cvmSet(cPoint2,0,i,(double)Left[i].x);
		cvmSet(cPoint2,1,i,(double)Left[i].y);
	}
	//CV_FM_LMEDS 
	//CV_FM_8POINT 
	//CV_FM_RANSAC
	/* Matrice fondamentale */

	if(!(*size>=8 && cvFindFundamentalMat(cPoint1,cPoint2,mat,CV_FM_RANSAC,1.0,0.99,status)))
	{
		fprintf(Out,"Matrice non trouvee\n");
		(*size)=0;
		return 0;
	}
	else
		fprintf(Out,"Matrix trouvee\n");
	matrix = malloc(sizeof(CvMatrix3));
	for(i=0;i<9;i++)
		matrix->m[i/3][i%3] = (float)cvmGet(mat,i/3,i%3);
	/* Find scanlines */
	cvMakeScanlines(matrix,cvSize(w,h),0,0,0,0,size);
	/* Calcultate scanlines coordinates by the matrix */
	if(scanline.scanline1)
		free(scanline.scanline1);
	if(scanline.scanline2)
		free(scanline.scanline2);
	if(scanline.scanlinev)
		free(scanline.scanlinev);
	if(scanline.lens1)
		free(scanline.lens1);
	if(scanline.lens2)
		free(scanline.lens2);
	scanline.scanline1 = calloc((*size)*2,sizeof(int)*4);
	scanline.scanline2 = calloc((*size)*2,sizeof(int)*4);
	scanline.scanlinev = calloc((*size)*2,sizeof(int)*4);
	scanline.lens1 = calloc((*size)*2,sizeof(int)*4);
	scanline.lens2 = calloc((*size)*2,sizeof(int)*4);
	cvMakeScanlines(matrix,cvSize(w,h),scanline.scanline1,scanline.scanline2,scanline.lens1,scanline.lens2,size);
	if(DEBUGP)
		printf("Scanlines trouvees : %d",*size);
	cvReleaseMat(&cPoint1);
	cvReleaseMat(&cPoint2);
	cvReleaseMat(&mat);
	cvReleaseMat(&status);
	return 1;
}
/* a=> 0 image droite, 1 image gauche */
//CvPoint2D64d* Convert(IplImage *img,int a,int size,CvPoint2D64d *tab)
CvPoint2D64d * Convert(IplImage *img,int a,int size,CvPoint2D64d *tab)
{
	int *Tab;
	int x1,y1,x2,y2,i,j;
//	FILE *file;
//	static CvPoint2D64d *tmp = NULL;
	
	tab = realloc(tab,sizeof(CvPoint2D64d)*size*4);

//	IplImage *r = cvCreateImage(cvSize(img->width*2,img->height*2),img->depth,img->nChannels);
//	cvResize(img,r,CV_INTER_NN);
	if(size<8)
		return NULL;
	if(a==1)
		Tab = scanline.scanline1;
	else
		Tab = scanline.scanline2;
//	file = fopen("point.txt","w");
//	fprintf(file,"Image : %d %d\n",img->height,img->width);
	/* Tous les scanlines */
    for (i = 0,j=0; i < size; i+=1,j+=2) 
	{
		x1 = Tab[i * 4    ];
        y1 = Tab[i * 4 + 1];
        x2 = Tab[i * 4 + 2];
        y2 = Tab[i * 4 + 3];
		tab[j  ].x = (float)x1;
		tab[j  ].y = (float)y1;
		tab[j+1].x = (float)x2;
		tab[j+1].y = (float)y2;
	}
//	fprintf(file,"Taille : %d\n",size);
//	for(i=0;i<size*2;i+=2)
//	{
//		Line(img,cvPoint((int)tmp[i].x,(int)tmp[i].y),cvPoint((int)tmp[i+1].x,(int)tmp[i+1].y));
//		fprintf(file,"(%d) %g %g -> %g %g\n",i,tmp[i].x,tmp[i].y,tmp[i+1].x,tmp[i+1].y);
//	}
//	fclose(file);
//	cvNamedWindow("_",1);
//	cvShowImage("_",r);
//	return tmp;
	return tab;
}

IplImage *Morphing(IplImage *image1,IplImage *image2,int numScanlines)
{
	static IplImage *imageRes=NULL,*r;
	int mem=0;
	static int nbscan = 0;
	CvSize     imgsize;

	if(scanline.morph_lens)
		free(scanline.morph_lens);
	if(scanline.buffer1)
		free(scanline.buffer1);
	if(scanline.buffer2)
		free(scanline.buffer2);
	if(scanline.morph_buffer)
		free(scanline.morph_buffer);
	if(scanline.runs1)
		free(scanline.runs1);
	if(scanline.runs2)
		free(scanline.runs2);
	if(scanline.corr1)
		free(scanline.corr1);
	if(scanline.corr2)
		free(scanline.corr2);
	if(scanline.num_runs1)
		free(scanline.num_runs1);
	if(scanline.num_runs2)
		free(scanline.num_runs2);
	imgsize.width   = image1->width*3;
	imgsize.height  = image1->height*3;
	scanline.morph_lens      = (int*)(calloc( numScanlines * 2, sizeof(int)*4));

	scanline.buffer1        =   (uchar*)(malloc(imgsize.width * (imgsize.height+1) * 3 * sizeof(uchar)));

	scanline.buffer2        = (uchar*)(malloc(imgsize.width * (imgsize.height+1) * 3 * sizeof(uchar)));

	scanline.morph_buffer    =	(uchar*)(calloc(imgsize.width * (imgsize.height+1), 3 * sizeof(uchar)));

	scanline.runs1          = (int*)(calloc(imgsize.width * imgsize.height * 2, 2 * sizeof(int)));
	scanline.runs2          = (int*)(calloc(imgsize.width * imgsize.height * 2, 2 * sizeof(int)));

	scanline.corr1          = (int*)(calloc(imgsize.width * imgsize.height * 2, 2 * sizeof(int)));
	scanline.corr2          = (int*)(calloc(imgsize.width * imgsize.height * 2, 2 * sizeof(int)));

	scanline.num_runs1      = (int*)(calloc(imgsize.width * imgsize.height * 2, 2 * sizeof(int)));
	scanline.num_runs2      = (int*)(calloc(imgsize.width * imgsize.height * 2, 2 * sizeof(int)));

	nbscan = numScanlines;
	print("warp");
	/* Prewarp first image */
	cvPreWarpImage( numScanlines,
					image1,
					scanline.buffer1,
					scanline.lens1,
					scanline.scanline1);
		r = cvCreateImage(cvGetSize(image1),image1->depth,image1->nChannels);
        
	cvSetZero(r);
	cvSetData (r, scanline.buffer1,image1->widthStep);
	cvNamedWindow("test",0);
	cvShowImage("test",r);


	/* Prewarp second image */
	cvPreWarpImage( numScanlines,
					image2,
					scanline.buffer2,
					scanline.lens2,
					scanline.scanline2);

	print("run");
	/* Create Runs on first Image */
	cvFindRuns(	numScanlines,
				scanline.buffer1,   /* prewarp image				*/
				scanline.buffer2,
				scanline.lens1,     /* prewarp line lens in pixels	*/
				scanline.lens2,
				scanline.runs1,     /* first runs					*/
				scanline.runs2,
				scanline.num_runs1, /* number of first runs			*/
				scanline.num_runs2);
	print("dynamic correspondance");
	cvDynamicCorrespondMulti(
								numScanlines,         // number of scanlines
								scanline.runs1,         // s0|w0|s1|w1|...
								scanline.num_runs1,    // numbers of runs
								scanline.runs2,
								scanline.num_runs2,
								scanline.corr1,    // s0'|e0'|s1'|e1'|...
								scanline.corr2 );

	print("alpha");
	cvMakeAlphaScanlines(   scanline.scanline1,
                            scanline.scanline2,
                            scanline.scanlinev,
                            scanline.morph_lens,
                            numScanlines,
                            0.5);				// 50%
	print("morphing");
    cvMorphEpilinesMulti(
                    numScanlines,			/* number of lines                              */
                    scanline.buffer1,       /* raster epilines from the first image         */
                    scanline.lens1,         /* number of pixel in first line                */
                    scanline.buffer2,       /* raster epilines from the second image        */
                    scanline.lens2,         /* number of pixel in second line               */
                    scanline.morph_buffer,  /* raster epiline from the destination image    */
											/* (it's an output parameter)                   */
                    scanline.morph_lens,    /* numbers of pixel in output line              */
                    0.5,						/* relative position of camera                  */
                    scanline.runs1,         /* first sequence of runs                       */
                    scanline.num_runs1,     /* it's length                                  */
                    scanline.runs2,         /* second sequence of runs                      */
                    scanline.num_runs2,     /*                                              */
                    scanline.corr1,         /* correspond information for the 1st seq       */
                    scanline.corr2);        /* correspond information for the 2nd seq       */
        


	print("image");
	if(!imageRes)
		imageRes  = cvCreateImage(cvGetSize(image1),image1->depth,image1->nChannels);
        
	cvSetZero(imageRes);

	print("post warp");
	cvPostWarpImage(
					numScanlines,                       /* number of scanlines  */
					scanline.morph_buffer,                   /* source buffers       */
					scanline.morph_lens,                     /* lens of buffers      */
					imageRes,       /* dest image           */
					scanline.scanlinev                     /* scanline             */
					);
	print("fin");
    cvDeleteMoire(  imageRes);
	return imageRes;
}

/*
		scanline.morph_lens      = (int*)(calloc( numScanlines * 2, sizeof(int)*4));
		mem += numScanlines * 2* sizeof(int)*4;
		printf("morph_lens : mmoire : %d\n",mem);

//		scanline.buffer1        =  (unsigned char*)(malloc(imgsize.width * (imgsize.height+1) * 3 * sizeof(unsigned char)));
//		scanline.buffer1        =  (unsigned char*)(malloc(MAX(image1->width ,image1->height)*numScanlines * 3 * sizeof(unsigned char)));
		scanline.buffer1        =  (unsigned char*)(malloc(image1->width*image2->height*sizeof(unsigned char)));
		mem += MAX(image1->width ,image1->height)*numScanlines * 3 * sizeof(unsigned char);
		printf("buffer1 : mmoire : %d\n",mem);

//		scanline.buffer2        =  (unsigned char*)(malloc(imgsize.width * (imgsize.height+1) * 3 * sizeof(unsigned char)));
//		scanline.buffer2        =  (unsigned char*)(malloc(MAX(image1->width ,image1->height)*numScanlines * 3 * sizeof(unsigned char)));
		scanline.buffer2        =  (unsigned char*)(malloc(image1->width*image2->height*sizeof(unsigned char)));
		mem += MAX(image1->width ,image1->height)*numScanlines * 3 * sizeof(unsigned char);
		printf("buffer2 : mmoire : %d\n",mem);

//		scanline.morph_buffer    = (unsigned char*)(calloc(imgsize.width * (imgsize.height+1), 3 * sizeof(unsigned char)));
//		scanline.morph_buffer    = (unsigned char*)(calloc(MAX(image1->width ,image1->height)*numScanlines * 3, 3 * sizeof(unsigned char)));
		scanline.morph_buffer    = (unsigned char*)(malloc(MAX(image1->width ,image1->height)*numScanlines * 3 * sizeof(unsigned char)));
		mem += MAX(image1->width ,image1->height)*numScanlines * 3 * sizeof(unsigned char);
		printf("morph_buffer : mmoire : %d\n",mem);

//		scanline.runs1          = (int*)(calloc(imgsize.width * imgsize.height * 2, 2 * sizeof(int)));
		scanline.runs1          = (int*)(malloc(MAX(image1->width ,image1->height)*numScanlines * 3* sizeof(int)));
//		scanline.runs1          = (int*)(malloc(numScanlines * 2 * sizeof(int)));
		mem +=MAX(image1->width ,image1->height)*numScanlines * 3* sizeof(int);
		printf("runs1 : mmoire : %d\n",mem);
//		scanline.runs2          = (int*)(calloc(imgsize.width * imgsize.height * 2, 2 * sizeof(int)));
		scanline.runs2          = (int*)(malloc(MAX(image1->width ,image1->height)*numScanlines*3* sizeof(int)));
//		scanline.runs2          = (int*)(malloc(numScanlines* 2 * sizeof(int)));
		mem +=MAX(image1->width ,image1->height)*numScanlines * 3* sizeof(int);
		printf("runs2 : mmoire : %d\n",mem);


//		scanline.corr1          = (int*)(calloc(imgsize.width * imgsize.height * 2, 2 * sizeof(int)));
//		scanline.corr1          = (int*)(calloc(MAX(image1->width ,image1->height)*numScanlines*3, 2 * sizeof(int)));
		scanline.corr1          = (int*)(malloc(MAX(image1->width ,image1->height)*numScanlines*3* sizeof(int)));
		mem +=MAX(image1->width ,image1->height)*numScanlines*3* sizeof(int);
		printf("corr1 : mmoire : %d\n",mem);
//		scanline.corr2          = (int*)(calloc(imgsize.width * imgsize.height * 2, 2 * sizeof(int)));
//		scanline.corr2          = (int*)(calloc(MAX(image1->width ,image1->height)*numScanlines*3, 2 * sizeof(int)));
		scanline.corr2          = (int*)(malloc(MAX(image1->width ,image1->height)*numScanlines*3 * sizeof(int)));
		mem +=MAX(image1->width ,image1->height)*numScanlines*3 * sizeof(int);
		printf("corr2 : mmoire : %d\n",mem);

//		scanline.num_runs1      = (int*)(calloc(imgsize.width * imgsize.height * 2, 2 * sizeof(int)));
		scanline.num_runs1      = (int*)(malloc(MAX(image1->width ,image1->height)*numScanlines*3* sizeof(int)));
//		scanline.num_runs1      = (int*)(malloc(numScanlines * 3 * sizeof(int)));
		mem += imgsize.width * imgsize.height * 2* 2 * sizeof(int);
		printf("num_runs1 : mmoire : %d\n",mem);
//		scanline.num_runs2      = (int*)(calloc(imgsize.width * imgsize.height * 2, 2 * sizeof(int)));
		scanline.num_runs2      = (int*)(malloc(MAX(image1->width ,image1->height)*numScanlines*3* sizeof(int)));
//		scanline.num_runs2      = (int*)(malloc(numScanlines * 3 * sizeof(int)));
		mem += MAX(image1->width ,image1->height)*numScanlines*3* sizeof(int)
		printf("num_runs2 : mmoire : %d\n",mem);
//		indic=1;
*/

