#include "tracker.h"

void InitCriteria(Tracker *tracker)
{
	tracker->quality	 = 0.01;
	tracker->minDistance = 1;
	tracker->level		 = 3;
}

void InitTracker(IplImage *img,Tracker *tracker,int *nbpoints,CvPoint3D64d **score)
{
	if( img && tracker)
	{
		int i;
		CvSize size;
		IplImage *temp[2];
		IplImage *img0;
		CvPoint2D32f *tmp;
		size.width = img->width;
		size.height = img->height;
		/* Allocation et creation des images NG */
		img0 = cvCreateImage(size,IPL_DEPTH_8U,1);
		for(i=0;i<2;i++)
		{
			temp[i] = cvCreateImage(size,IPL_DEPTH_32F,3);
			temp[i]->depth = IPL_DEPTH_32F;
			temp[i]->nChannels = 1;
		}
		(*score) = (CvPoint3D64d*)malloc(sizeof(CvPoint3D64d)*(*nbpoints));
		tracker->point = (CvPoint2D64d*)malloc((*nbpoints)*sizeof(CvPoint2D64d));
		tmp = (CvPoint2D32f*)malloc((*nbpoints)*sizeof(CvPoint2D32f));
		/* Conversion en NG */
		cvCvtColor(img,img0,CV_BGR2GRAY);
		/* Initialisation du tracker */
		cvGoodFeaturesToTrack(img0,temp[0],temp[1],tmp,nbpoints,0.1,10,0);
		/* Affinement des points */
		if( (*nbpoints) > 0 )
		{
			cvFindCornerSubPix( img0, tmp, *nbpoints, 
								cvSize(5,5), cvSize(-1,-1),
								cvTermCriteria( CV_TERMCRIT_ITER, 10, 0.1f ));
			tracker->count = *nbpoints;
			fprintf(Out,"Tracking \n");
			for(i=0;i<tracker->count ; i++)
			{
				tracker->point[i].x = tmp[i].x;
				tracker->point[i].y = tmp[i].y;
				fprintf(Out,"(%d) %g %g\n",i,tracker->point[i].x,tracker->point[i].y);
				(*score)[i].z = 1;
			}
		}
		fflush(Out);
		/* Libration de la mmoire */
		cvReleaseImage(&img0);
		for(i=0;i<2;i++)
			cvReleaseImage(&temp[i]);
		free(tmp);
	}
}
void Track(IplImage *img,Tracker *tracker0,CvPoint3D64d **score0,IplImage *img1,Tracker *tracker1,CvPoint3D64d **score1)
{
	if( tracker0->count > 0 && img )
	{
		int i;
		int pyr0_ready = 0;
		IplImage *frame1,*frame0,*pyr1,*pyr0;
		char *status;
		CvPoint2D32f *tmp0,*tmp1;
		CvSize size;
		size.width = img->width;
		size.height = img->height;
		/* Allocation et Cration des images en NG */
		frame0 = cvCreateImage(size,IPL_DEPTH_8U,1);
		frame1 = cvCreateImage(size,IPL_DEPTH_8U,1);
		pyr0 = cvCreateImage(size,IPL_DEPTH_8U,1);
		pyr1 = cvCreateImage(size,IPL_DEPTH_8U,1);
		tracker1->point = (CvPoint2D64d*)malloc(tracker0->count*sizeof(CvPoint2D64d));
		tmp0 = (CvPoint2D32f*)malloc(tracker0->count*sizeof(CvPoint2D32f));
		tmp1 = (CvPoint2D32f*)malloc(tracker0->count*sizeof(CvPoint2D32f));
		status = (char*)malloc(sizeof(char)*tracker0->count);
		(*score1) = (CvPoint3D64d*)malloc(sizeof(CvPoint3D64d)*tracker0->count);
		if(!*score0)
			(*score0) = (CvPoint3D64d*)malloc(sizeof(CvPoint3D64d)*tracker0->count);
		/* Conversion NB */
		cvCvtColor(img1,frame1,CV_BGR2GRAY);
		cvCvtColor(img,frame0,CV_BGR2GRAY);
		/* Rcupration des points */
		for(i=0;i<tracker0->count;i++)
		{
			tmp0[i].x = (float)tracker0->point[i].x;
			tmp0[i].y = (float)tracker0->point[i].y;
		}
		/* Suivi des points */
		 cvCalcOpticalFlowPyrLK( frame0, frame1, pyr0, pyr1,
								 tmp0,tmp1,
                                 tracker0->count, cvSize(10,10), 3,
                                 status, 0, 
                                 cvTermCriteria(CV_TERMCRIT_ITER|CV_TERMCRIT_EPS,20,0.03),
                                 pyr0_ready ? CV_LKFLOW_PYR_A_READY : 0 );
		tracker1->count = tracker0->count;
		/* Mise  jour des status des points */
		fprintf(Out,"Suivi\n");
        for( i = 0 ; i < tracker0->count; i++ )
		{
			tracker1->point[i].x = tmp1[i].x;
			tracker1->point[i].y = tmp1[i].y;
			fprintf(Out,"(%d) %g %g\n",i,tracker1->point[i].x,tracker1->point[i].y);
			(*score1)[i].z = status[i];
			if((*score0)[i].z != 1 || (*score1)[i].z != 1 )
			{
				(*score0)[i].z = -1;
				(*score1)[i].z = -1;
				fprintf(Out,"%d perdu\n",i);
			}
		}
		fflush(Out);
		/* libration mmoire */
		free(tmp0);
		free(tmp1);
		free(status);
		cvReleaseImage(&frame0);
		cvReleaseImage(&frame1);
		cvReleaseImage(&pyr0);
		cvReleaseImage(&pyr1);
    }
}
/* Distance entre deux points */
double Distance(double x1,double y1,double x2,double y2)
{
	return sqrt( (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2));
}

/* Comparaison de deux tableaux ordonns (images identiques) */
void ComparaisonV2(CvPoint2D64d *point1,CvPoint3D64d *score0,CvPoint2D64d *point2,CvPoint3D64d *score1,int size,double seuil)
{
	int i;
	double d;
	double trig = 1.6;
	fprintf(Out,"Comparaison version 2\n");
	/* Recherche des correspondances */
	for(i=0;i<size;i++)
	{
		if(score0[i].z != 1 || score1[i].z != 1 )
			continue;
		d = Distance(point1[i].x,point1[i].y,point2[i].x,point2[i].y);
		if( d <= seuil)
		{
			score0[i].x =score1[i].x = i;
			score0[i].y =score1[i].y = i;
			score0[i].z =score1[i].z += d;
			fprintf(Out,"%d %g\n",i,d);
		}
	}
	fflush(Out);
}
/* Comparaison de deux tableau non-ordonns (images identiques */
void ComparaisonV3(CvPoint2D64d *point1,CvPoint3D64d *score1,int size1,CvPoint2D64d *point2,CvPoint3D64d *score2,int size2,double seuil)
{
	int i,j,k;
	double d;
	fprintf(Out,"Comparaison version 3\n");
	/* Pour la premiere image */
	for(i=0;i<size1;i++)
	{
		/* Pour la seconde image */
		for(j=0;j<size2;j++)
		{
			d = Distance(point1[i].x,point1[i].y,point2[j].x,point2[j].y);
			/* Verification d'un point proche */
			if(d<seuil)
			{
				/* Verification d'un point plus proche */
				if( score1[i].z >= 1 && score1[i].z <= 1+seuil && seuil > d)
				{
					/* Recherche du point dj existant et rinitialisation dans la premiere image */
					if(score1[i].z >1)
					{
						for(k=0 ; k<j ; k++)
						{
							if(score2[k].z == score1[i].z)
								score2[k].z=1;
						}
						score1[i].z = 1;
					}
					/* Recherche du point dj existant et rinitialisation dans la seconde image */
					if(score2[j].z > 1)
					{
						for(k=0 ; k<i ; k++)
						{
							if(score1[k].z == score2[j].z)
								score1[k].z=1;
						}
						score2[j].z = 1;
					}
					/* Affectation du score (distance) des points des deux images */
					score1[i].z = 1+d;
					score2[j].z = 1+d;
				}
			}
		}
	}
//	for(i=0;i<size;i++)
//			fprintf(Out,"%d %g %g\n",i,score1[i].z,score2[i].z);
	fflush(Out);
}

/* Comparaison des indices */
void ComparaisonV4(CvPoint3D64d *scoreL0, CvPoint3D64d *scoreL1, CvPoint3D64d *scoreR0, CvPoint3D64d *scoreR1, int size1, int size2, double seuil)
{
	int i,j;
	fprintf(Out,"Comparaison version 4\n");
	/* Recherche des couples dans la partie gauche*/
	for(i=0;i<size1;i++)
	{
		for(j=0;j<size2;j++)
		{
			if((scoreL0[i].z > 1 || scoreL1[j].z >1) && (scoreR0[i].z > 1 || scoreR1[j].z >1) && (scoreL0[i].z == scoreL1[j].z))
			{
				fprintf(Out,"couple : %d %d (%g,%g %g,%g) ",i,j,scoreL0[i].z,scoreL1[j].z,scoreR0[i].z,scoreR1[j].z);
				if(scoreR0[i].z == scoreR1[j].z)
				{
					fprintf(Out," ==>verifier");
					scoreL1[j].z += seuil;
					scoreR1[j].z += seuil;
				}
				fprintf(Out,"\n");
			}
		}
	}
	fflush(Out);
}
